
# dobserv_funcs.r - Double-observer functions

logx <- function(x) {  x[x<=0]=1e-12; return(log(x))}

pool_lt10 <- function(dobsdata) {
  #' Pool species data with < 10 detections per species
  #'
  #' \code{pool_lt10} pools species w/ < 10 detections
  #'
  #'@param dobsdata input data-matrix with 4 columns:
  #' \itemize{
  #' \item{observerNumber - number of observer who is primary observer}
  #' \item{species - char string with species name}
  #' \item{N1 - number of individuals of species seen by primary observer}
  #' \item{N2 - number of individuals of species seen by secondary observer which were missed by primary observer}
  #' }
  #'
  #'@return data matrix w/ speciesName replaced if < 10 detections
  #'@export
  #'@author Jim Hines
  #'@examples
  #' \dontrun{
  #' fname=system.file('extdata','sample.csv',package='dobserv')
  #' data=read.csv(fname,header=F,skip=1)
  #' pooled_data=pool_lt10(data)
  #' }
  #'
  #  read input file and summarize number of detections by species...
  sp=spx=as.character(dobsdata[,2]);
  a1=aggregate(dobsdata[,3]+dobsdata[,4],by=list(dobsdata[,2]),sum)
  #  get species with less than 10 detections
  i=which(a1$x<10); sp9=as.character(a1$Group.1[i])
  #  for each species w/ < 10 detections, replace species name w/ 'grp1'
  for (j in 1:length(sp)) if (as.character(sp[j]) %in% sp9) spx[j]='grp1'
  return(data.frame(obs=dobsdata[,1],sp=spx,X1=dobsdata[,3],X01=dobsdata[,4]))
}

like_dep <- function(theta,d,dm) {
  #'  likelihood function for dependent observers
  #'
  #'  \code{like_dep} returns likelihood value for parameter vector theta,
  #'  data matrix d, and design matrix, dm
  #'
  #'@param theta vector of parameters (detection probs)
  #'@param d data matrix
  #'@param dm design matrix
  #'
  #'@return negative log-likelihood value
  #'
  #'@author Jim Hines
  #'
  #'@seealso like_indep

  th=plogis(dm %*% theta); nreal=nrow(dm); p1=th[seq(1,nreal,2)]; p2=th[seq(2,nreal,2)]
  denom=p1+p2-p1*p2; prb1=p1/denom;     prb2=(1-p2)*p1/denom
  like=d$X1*logx(prb1)+d$X01*logx(1-prb1)+d$Y1*logx(prb2)+d$Y01*logx(1-prb2)
  n1=rowSums(cbind(d$X1,d$X01)); n2=rowSums(cbind(d$Y1,d$Y01))
  expval<<-cbind(n1*prb1,n1*(1-prb1),n2*prb2,n2*(1-prb2))
  #cat('like',-sum(like),'\n')
  return(-sum(like))
}
like_indep <- function(theta,d,dm) {
  #'  likelihood function for independent observers
  #'
  #'  \code{like_indep} returns likelihood value for parameter vector theta,
  #'  data matrix d, and design matrix, dm
  #'
  #'@param theta vector of parameters (detection probs)
  #'@param d data matrix
  #'@param dm design matrix
  #'
  #'@return negative log-likelihood value
  #'
  #'@author Jim Hines
  #'
  #'@seealso like_indep
  #'

  th=plogis(dm %*% theta); nreal=nrow(dm); thj=th[seq(1,nreal,2)]; thjp1=th[seq(2,nreal,2)]
  denom=1-(1-thj)*(1-thjp1); p1=thj*(1-thjp1)/denom; p2=(1-thj)*thjp1/denom
  like=sum(d[,1]*log(p1)+d[,2]*log(p2)+d[,3]*log(1-p1-p2))
  n1=rowSums(cbind(d$X10,d$X01,d$X11))
  expval<<-cbind(n1*p1,n1*p2,n1*(1-p1-p2))
  return(-like)
}
invhess <- function(h,est) {
  #'  invert hessian matrix
  #'
  #'  \code{invhess} inverts hessian matrix from optmiz function
  #'  deleting rows/cols of all zeros if necessary
  #'  and inserting zeros into result
  #'@param h input hessian matrix
  #'@return inverse matrix
  #'
  i=which(rowSums(h)==0); h2=h3=matrix(0,nrow(h),ncol(h))
  i=which(abs(est)>15)
  if (length(i)==0) {
      res=try(h3<-solve(h))
  } else {
      res=try(h2<-solve(h[-i,-i]),T); k=0
      if (class(res)!='try-error') {
        for (j in 1:ncol(h)) {
          if (!(j %in% i)) {
            k=k+1; a=rep(0,ncol(h)); a[-i]=h2[,k]; h3[,j]=a; h3[j,]=a
          }
        }
      }
  }
  return(h3)
}
runmod <- function(frm,mx,d,type='dependent') {
  #'  runs double-observer model
  #'
  #'  \code{runmod} run doble-obs model
  #'
  #'@param frm formula for model (eg., ~obs)
  #'@param mx species list
  #'@param d data frame (sp,obs,x1,x2 for dep; sp,obs,x11,x10,x01 for indep)
  #'@param type "dependent" or "independent"
  #'
  #'@return table of model results
  #'@export
  #'
  #'@author Jim Hines
  #'
  #'@seealso dobserv_dep_obs
  #'
  #'@examples
  #'#
  #' \dontrun{
  #'  fname=system.file('extdata','sample2.csv',package='dobserv')
  #'  z1=dobserv_indep_obs(fname=fname,mods=c(~sp,~obs,~sp*obs,~sp+obs))
  #'  }

  ##  Example data2 - sample2.csv : example using simulated data w/ "independent" observers

  cat('model:p(',gsub(' ','',as.character(frm)),')\n',sep='')
  dm=model.matrix(frm,mx); th.init=matrix(2,ncol(dm),1); nreal=nrow(mx)
  like=like_dep; ii=2; if (type!='dependent') { ii=3; like=like_indep}
  t1a=system.time(m1a<-nlm(like,p=th.init,d=d,dm=dm,hessian=T,iterlim=500))
  m1a=m1b=list(minimum=1e22);
  try(t1b<-system.time(m1b<-nlm(like,p=th.init-th.init,d=d,dm=dm,hessian=T,iterlim=500)),silent=T)
  if (m1b$minimum<m1a$minimum) m1a=m1b
  m1a$name=paste0('p(',as.character(frm[-1]),')',collapse=''); m1a$type=type
  m1a$dm=dm; vcbeta=matrix(NA,length(m1a$estimate),length(m1a$estimate))
  res=try(vcbeta<-invhess(m1a$hessian,m1a$estimate)); #res=try(vcbeta<-solve(m1a$hessian),TRUE);
  nbeta=ncol(vcbeta); grad=matrix(0,nreal,nbeta); grad2=gradN=matrix(0,nreal/2,nbeta); gradTN=matrix(0,1,nbeta)
  beta1=m1a$estimate; p=plogis(dm %*% beta1);
  tmp=matrix(p,ncol=2,byrow=T); pstar=tmp[,1]+tmp[,2]-tmp[,1]*tmp[,2]
  i=grep("^[XY]",colnames(d)); dd=d[,i]
  xdotdot=sum(dd)
  if (nrow(d)>1) xdotdot=rowSums(dd);
  N=xdotdot/pstar; totN=sum(N); eps1=.1e-12
  for (k in 1:nbeta) {
    beta1[k]=beta1[k]+eps1; p2=plogis(dm %*% beta1);
    tmp=matrix(p2,ncol=2,byrow=T); pstar2=tmp[,1]+tmp[,2]-tmp[,1]*tmp[,2]; N2=xdotdot/pstar2; totN2=sum(N2)
    grad[,k]=(p2-p)/eps1; grad2[,k]=(pstar2-pstar)/eps1; gradN[,k]=(N2-N)/eps1; gradTN[k]=(totN2-totN)/eps1
    beta1[k]=beta1[k]-eps1
  }
  vcp=matrix(NA,nreal,nreal); vcpstar=vcN=matrix(NA,nreal/2,nreal/2)
  if (class(res)=='try-error')  {
    cat('*********************\nerror w/ hessian for model p(',as.character(frm),')\n************************\n',sep='')
  } else  {
    vcp=grad %*% vcbeta %*% t(grad); vcpstar=grad2 %*% vcbeta %*% t(grad2)
    vcN=gradN %*% vcbeta %*% t(gradN); vcTN=gradTN %*% vcbeta %*% t(gradTN)
  }
  varp=diag(vcp); sep=sqrt(varp); sepstar=sqrt(diag(vcpstar));
  nhat=xdotdot/pstar; seTN=sqrt(vcTN)
  m1a$totN=totN; m1a$seTN=seTN
  #varnhat=nhat^2/pstar^2 + xdotdot*(1-pstar)/pstar^2; senhat = sqrt(varnhat)
  varnhat=diag(vcN) + xdotdot*(1-pstar)/pstar^2; senhat = sqrt(varnhat)
  m1a$res=res; m1a$cputime=t1a; m1a$vcbeta=vcbeta; m1a$loglik=m1a$minimum; m1a$aic=2*m1a$minimum+2*ncol(m1a$dm)
  p=matrix(p,ncol=2,byrow=T); sep=matrix(sep,ncol=2,byrow=T)
  f0 = nhat - xdotdot; f0[f0==0]=.1e-22
  c = rep(.1e-22,length(varnhat)); i=which(varnhat>0)
  res=try(c[i] <- exp(1.96 * sqrt(log(1 + varnhat[i] / f0[i]^2))),TRUE)
  CI1 = xdotdot+f0 / c; CI2 = xdotdot+f0*c; CI2[CI2<CI1]=CI1[CI2<CI1]
  z=cbind(xdotdot,p[,1],sep[,1],p[,2],sep[,2],pstar,sepstar,nhat,senhat,CI1,CI2)
  colnames(z)=c('X..','p1','se(p1)','p2','se(p2)','p*','se(p*)','N','se(N)','lowCI','hiCI'); m1a$z=z
  m1a$ntotal=compute_var_sum_N(rep(1,nrow(d)),m1a,d,rep(1,nrow(d)))
  names(m1a$ntotal)=c('X..','p*','se(p*)','N','se(N)','lowCI','hiCI')
  return(m1a)
}
compute_var_sum_N <- function(specx,m,d,speclist) {
  theta=m$estimate; th=plogis(m$dm %*% theta); nreal=nrow(m$dm)
  thj=th[seq(1,nreal,2)]; thjp1=th[seq(2,nreal,2)]; denom=thj+thjp1-thj*thjp1;
  j=grep("^[XY]",colnames(d)); dd=cbind(d[,j])
  k=which(speclist==specx); if (length(k)==1) x=sum(dd[k,]) else x=rowSums(dd[k,])  # x=x.. for each species
  p=denom[k]; N=sum(x/p); pstar=sum(x)/N; dndth=dpdth=NULL;
  tmp=sum(x*(1-p)/p^2)
  for (j in 1:length(theta)) {
    theta[j]=theta[j]+1e-12
    th=plogis(m$dm %*% theta); thj=th[seq(1,nreal,2)]; thjp1=th[seq(2,nreal,2)]; denom=thj+thjp1-thj*thjp1;
    p1=denom[k]; N1=sum(x/p1); dndth=cbind(dndth,(N1-N)/1e-12);
    pstar1=sum(x)/N1; dpdth=cbind(dpdth,(pstar1-pstar)/1e-12)
    theta[j]=theta[j]-1e-12
  }
  varn=dndth %*% m$vcbeta %*% t(dndth); sumx=sum(x); varnhat=varn+tmp
  varp=dpdth %*% m$vcbeta %*% t(dpdth)
  f0 = N - sumx; if (f0==0) f0=.1e-22
  c <- exp(1.96 * sqrt(log(1 + varnhat / f0^2)))
  CI1 = sumx+f0 / c; CI2 = sumx+f0*c
  seN=sqrt(varn+tmp)
  #if (sum(p<1e-5)>0) N=seN=CI1=CI2=NA
  return(c(sum(x),pstar,sqrt(varp),N,seN,CI1,CI2))
}
print_by_spec <- function(m,speclist,d) {
  #'  print estimates of p, summarized by species
  #'
  #'  \code{print_by_species} returns table of estimates, summarized by species
  #'  (or some other variable)
  #'
  #'@param m list with model output
  #'@param speclist list of all possible species to summarize by
  #'@param d data matrix of detections
  #'
  #'@return table of estimates of N, p, by species
  #'@export
  #'
  #'@author Jim Hines
  #'
  #'@examples
  #'#
  #'  fname=file('sample.csv',package=dobserv)
  #'  z1=dobserv_dep_obs(fname,mods=c(~obs,~-1+sp,~sp+obs,~sp*obs));
  #'  m=z1$m$"p(sp * obs)"
  #'  cat('\nN by TRT\n');
  #'  n1=print_by_spec(m,as.character(z1$data$SPECIES),  z1$d)
  specx=levels(as.factor(speclist));
  if (ncol(d)<5) d=cbind(d,0)
  n=round(t(sapply(specx,compute_var_sum_N,m,d[,3:5],speclist)),4);
  n[,4:7]=round(n[,4:7],2);
  colnames(n)=c('X..','p*','se(p*)','N','se(N)','low95%CI','hi95%CI')
  i=which(n[,2]<1e-9); n[i,4:7]=NA
  return(n)
}
dobserv_dep_obs <- function(dobs_data,mods=c(~obs,~sp,~sp+obs,~sp*obs)) {
  #'  runs double-observer analysis for dependent observers
  #'
  #'  \code{dobserv_dep_obs} returns aic table of results
  #'
  #'@param dobs_data data frame with the following fields:\cr
  #'\itemize{
  #'  \item \code{obs} observer number for primary observer
  #'  \item \code{species} name of species
  #'  \item \code{X1.} number of individuals of species detected by primary observer
  #'  \item \code{X01} number of individuals of species detected by secondary
  #'    observer that were missed by primary observer
  #'  }
  #'@param mods list of model definitions. Model definitions are in GLM form.\cr
  #'  \itemize{
  #'  \item \code{~1} means detection is constant for all observers and species.\cr
  #'  \item \code{~sp} means detection varies by species (constant across observers)\cr
  #'  \item \code{~obs} means detection varies by observer (constant across species)\cr
  #'  \item \code{~sp*obs} means detection varies by species and observer\cr
  #'  \item \code{~xxxx} means detection varies by input covariate, xxxx\cr
  #'}
  #'@return aic table of model results
  #'@export
  #'
  #'@author Jim Hines
  #'
  #'@seealso dobserv_indep_obs
  #'
  #'@examples
  #'#
  #'fname=system.file('extdata','sample.csv',package='dobserv')

  #'data=pool_lt10(fname); d1=read.csv(fname,as.is=T)
  #'#   pool CAGO,PUMA even though > 10 detections because all detections were by same obs.
  #'i=which(data$sp=="CAGO" | data$sp=="PUMA"); data$sp[i]='grp1'
  #'
  #'#  summarize by species...
  #'sp=as.factor(data$sp); usp=levels(sp); nsp=length(usp); d2=matrix(0,2*nsp,2)
  #'jj=(as.numeric(sp)-1)*2+(data[,1]==data[1,1])+1
  #'for (i in 1:nrow(data)) d2[jj[i],]=d2[jj[i],]+as.numeric(data[i,3:4])
  #'d2=data.frame(obs=rep(1:2,nsp),sp=rep(as.character(usp),each=2),X1=d2[,1],X01=d2[,2])
  #'
  #'  #   run some models...
  #'    #    ~obs means detection depends only on observer,
  #'    #    ~sp means detection depends only on species,
  #'    #    ~xxxx means detection depends on input covariate xxxx
  #'
  #'z1=dobserv_dep_obs(d2,mods=c(~obs,~-1+sp,~sp+obs,~sp*obs))
  #'#   z1 is a list with output from each of the models above (plus default model, ~1)
  #'
  #'#  print results from model, p(sp*obs) (ie., detection different for each
  #'#                                         species and observer)
  #'print(round(z1$m$`p(sp * obs)`$z,4))

    ##  Example data - sample.csv : example using BBS data w/ "dependent" observers
                                                 ##      spec3
  i=which(dobs_data$obs==dobs_data$obs[1])
  d=dobs_data[i,]; d$Y1=0; d$Y01=0;
  if (length(i)==nrow(dobs_data)) {   #  if only 1 primary observer...
    speclist=as.character(dobs_data$sp)  #  tack on zeros for 2nd observer
  } else {
    dx1=dobs_data[i,]; dx2=dobs_data[-i,]
    for (j in 1:nrow(dx2)) {
      k=which(dx2$sp[j]==dx1$sp); d$Y1[k]=dx2$X01[j]; d$Y01[k]=dx2$X1[j] #d[k,4:3]=dx2[j,3:4]
    }
    speclist=as.character(dx1$sp); rownames(d)=speclist
    #d=rbind(cbind(as.matrix(dobs_data[i,3:4]),0,0),cbind(0,0,as.matrix(dobs_data[-i,4:3])))
    #speclist=c(as.character(dobs_data$sp[i]),as.character(dobs_data$sp[-i]))
  }
  ###colnames(d)=c('X1','X01','Y1','Y01')
  ##     where n1=# detected by obs1, obs1=primary
  ##           n2=# detected by obs2, missed by obs1, obs1=primary
  ##           n3=# detected by obs2, obs2=primary
  ##           n4=# detected by obs1, missed by obs2, obs2=primary

  nreal=2*nrow(d);    # no. real params = 2*(# sites) ... p(obs1),p(obs2) for each site
  obs=1-(1:nreal%%2); # obs covariate indicates which observer is primary for each site
  sp=rep(speclist,each=2)  # sp covariate = categorical species covariate

  ##          create design-data matrix
  mx=data.frame(int=rep(1,nreal),obs=obs,sp=as.factor(sp))
  ##          add covariates from input csv file, if input
  if (ncol(dobs_data)>=5) {
    print(head(mx)); cat('dim(mx)=',dim(mx),'\ndim(a)=',dim(dobs_data),'\n'); ncolmx=ncol(mx)
    for (i in 5:ncol(dobs_data)) mx=cbind(mx,rep(dobs_data[,i],each=2))
    colnames(mx)[(ncolmx+1):ncol(mx)]=colnames(dobs_data[5:ncol(dobs_data)])
  }
  i=grep("^[XY]",colnames(d)); dd=d[,i]
  ##          run a set of models
  cat('\n'); m=list(); i=0;
  for (s in c(~1,mods)) {
    i=i+1; m[[i]]=runmod(s,mx,d); names(m)[i]=m[[i]]$name
    j=which(dd>0,arr.ind=T);
    m[[i]]$G=sum(2*log(dd[j]/(expval[j]+1e-9))*dd[j])
  }

  ##          create AIC table of model results
  atbl=aictbl(m,d); print(atbl$table,quote=F)

  ##          print estimtes of N from "Best" model
  i=which(atbl$table[1,1]==names(m));     # i=index of model w/ lowest AIC in aictable

  n1=print_by_spec(m[[i]],speclist,d)
  cat('\nOverall\n'); print(round(m[[i]]$ntotal,2))

  return(list(data=dobs_data,d=d,m=m,aictbl=atbl))
}

dobserv_indep_obs <- function(dobsdata,mods=c(~obs)) {
  #'  runs double-observer analysis for independent observers
  #'
  #'  \code{dobserv_indep_obs} returns aic table of results
  #'
  #'@param dobs_data data frame with the following fields:\cr
  #'\itemize{
  #'  \item \code{obs} observer number for primary observer
  #'  \item \code{species} name of species
  #'  \item \code{X10} number of individuals of species detected by observer 1
  #'  and missed by observer 2
  #'  \item \code{X01} number of individuals of species missed by observer 1
  #'  and detected by observer 2
  #'  \item \code{X11} number of individuals of species detected by both observers
  #'  }
  #'@param mods list of model definitions. Model definitions are in GLM form.\cr
  #'  \itemize{
  #'  \item \code{~1} means detection is constant for all observers and species.\cr
  #'  \item \code{~sp} means detection varies by species (constant across observers)\cr
  #'  \item \code{~obs} means detection varies by observer (constant across species)\cr
  #'  \item \code{~sp*obs} means detection varies by species and observer\cr
  #'  \item \code{~xxxx} means detection varies by input covariate, xxxx\cr
  #'}
  #'
  #'@return aic table of model results
  #'@export
  #'
  #'@author Jim Hines
  #'
  #'@seealso dobserv_dep_obs
  #'
  #'@examples
  #'#
  #' \dontrun{
  #'
  #'fname=system.file('extdata','sample3.csv',package='dobserv')
  #'
  #'  #  read sample data file which includes a covariate (temperature) in the last column
  #'d3=read.csv(fname,header=F,skip=1); colnames(d3)=c('obs','sp','X10','X01','X11','temp')
  #'  #  run a few models...
  #'z2=dobserv_indep_obs(d3,mods=c(~obs,~temp*obs))
  #'  #  print estimates for each species...
  #'print(z2$m$`p(temp * obs)`$z)
  #'  #  print estimate of total number of individuals...
  #'print(z2$m$`p(temp * obs)`$ntotal)
  #'  }
  sp=as.character(dobsdata$sp);
  d=cbind(dobsdata$X10,dobsdata$X01,dobsdata$X11)
  nreal=2*nrow(d); colnames(d)=c('X10','X01','X11'); d=data.frame(d)
  mx=data.frame(int=rep(1,nreal),obs=1-(1:nreal%%2),sp=as.factor(rep(sp,each=2)))

  if (ncol(dobsdata)>=6) {
    print(head(mx)); cat('dim(mx)=',dim(mx),'\ndim(dobsdata)=',dim(dobsdata),'\n')
    for (i in 6:ncol(dobsdata)) mx=cbind(mx,rep(dobsdata[,i],each=2))
    colnames(mx)[4:ncol(mx)]=colnames(dobsdata)[6:ncol(dobsdata)]
  }
  m=list(); i=0;
  for (s in c(~1,mods)) {
    rv=try(mm<-runmod(s,mx,d,'independent'))
    if (!class(rv)=="try-error") {
      i=i+1; m[[i]]=mm; names(m)[i]=m[[i]]$name
      j=which(d>0,arr.ind=T);
      m[[i]]$G=sum(2*log(d[j]/(expval[j]+1e-9))*d[j])
    }
  }

  ##          create AIC table of model results
  atbl=aictbl(m,d); print(atbl$table,quote=F)

  i=which(atbl$table[1,1]==names(m));     # i=index of model w/ lowest AIC in aictable
  #cat('\nEstimates from top model -',m[[i]]$name,'\n')
  #print(m[[i]]$z,quote=F)
  cat('\ntotal N = ',m[[i]]$totN,' SE(N)=',m[[i]]$seTN,'\n')
  return(list(data=dobsdata,d=d,m=m,aictbl=atbl))
}

aictbl <- function(m,d) {
  #'  computes AIC table from model-list, m
  #'
  #'  \code{aictbl} returns aic table of results
  #'
  #'@param m list of models
  #'
  #'@return aic table of model results
  #'
  #'@author Jim Hines
  #'
  #'@seealso dobserv_indep_obs
  #'
  #'@examples
  #'#
  #' \dontrun{
  #'z1=dobserv_dep_obs(fname='sample.csv',mods=c(~sp,~obs,~sp*obs,~sp+obs))
  #'  }
  aictbl=NULL; j=grep("^[XY]",colnames(d)); dd=d[,j]
  ness=sum(dd); maxpar=j=0; K=sapply(1:length(m),function(i)length(m[[i]]$estimate))
  for (i in 1:length(m)) if (K[i]>maxpar) {maxpar=K[i]; j=i}
  chat=1; Gdf=(2*nrow(d)-(maxpar-1)+1);
  if (m[[j]]$G>0 & Gdf>0) chat=m[[j]]$G/Gdf;  llconst=0; if (chat<1) chat=1
  if (m[[1]]$type=="dependent") llconst=sum(lgamma(c(rowSums(dd[,1:2]),rowSums(dd[,3:4]))+1))-sum(lgamma(dd+1))
  if (m[[1]]$type=="independent") llconst=sum(lgamma(rowSums(dd)+1))-sum(lgamma(dd+1))
  cat('c-hat=',chat,'ness=',ness,'llconst=',llconst,'\n');
  for (i in 1:length(m)) {
    loglik=m[[i]]$minimum-llconst
    m[[i]]$aic=2*loglik+2*K[i]
    m[[i]]$aicc=2*loglik + 2*K[i] + 2*K[i]*(K[i] + 1)/(ness - K[i] - 1)
    m[[i]]$qaicc=-2*loglik/chat + 2*K[i] + 2*K[i]*(K[i] + 1)/(ness - K[i] - 1)
    aictbl=rbind(aictbl,cbind(m[[i]]$name,K[i],round(loglik,4),round(m[[i]]$aicc,4)))
  }
  aic=as.numeric(aictbl[,4]); dlta=aic-min(aic); tmp=exp(-dlta/2); aicwgt=tmp/sum(tmp)
  aictbl=cbind(aictbl,round(dlta,4),round(aicwgt,4))
  colnames(aictbl)=c('name','npar','negLogLike','AICc','deltaAICc','AICcwgt')
  if (chat>1) colnames(aictbl)=gsub("AIC","QAIC",colnames(aictbl))
  o=order(as.numeric(aictbl[,5]))
  return(list(table=aictbl[o,],chat=chat))
}

