% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dobserv-package.r
\docType{package}
\name{dobserv-package}
\alias{dobserv}
\alias{dobserv-package}
\title{DOBSERV Double-observer population estimation}
\description{
Computes estimates of population size using double-observer detection
history data.  .
A Double-observer approach for estimating detection probability and abundance
from avian point counts as described in Nichols et. al. (2000).\cr\cr
\cite{Nichols, J. D., J. E. Hines, J. R. Sauer, F. W. Fallon, J. E. Fallon, and P. J. Geglund.
A Double-observer Approach for Estimating Detection Probability and Abundance from Point.  Counts.
The Auk 2000; 117(2): 393-408.}
}
\details{
Although point counts are frequently used in ornithological studies, basic assumptions about
consistency in detection probabilities often remain untested. We apply a double-observer approach
developed to estimate detection probabilities for aerial surveys (Cook and Jacobson 1979) to avian
point counts. At each point count, one observer is designated "primary", and indicates to the
other ("secondary") observer all birds detected. The secondary observer records all detections
of the primary observer, as well as any birds not detected by the primary observer. Observers
alternate primary and secondary roles during the course of a survey. The approach permits
estimation of observer-specific detection probabilities and bird abundance. We developed a set
of models incorporating different assumptions about sources of variation (e.g., observer, bird
species) in detection probability. Single observer point counts generally miss varying proportions
of the birds actually present, as observer and bird species were both found to be relevant sources
of variation in detection probabilities. However, using the double observer approach, overall
detection probabilities (probability of being detected by at least 1 of the 2 observers) were very
high (>0.95) yielding precise estimates of avian abundance. We believe that most questions meriting
the effort required to carry out point counts also merit serious attempts to estimate detection
probabilities associated with the counts. The double-observer approach is a method which can be
used for this purpose.

\strong{Data input (Dependent observers)}

The input file for program DOBSERV consists of counts, by species, of individuals observed by
both observers, and counts of individuals observed by the secondary observer which were not
observed by the primary observer. The input file should be a "csv" file
 in the following form:

\code{O1,SPEC1a, N11a, N21a\cr
O1,SPEC1b, N11b, N21b\cr
O1,SPEC1c, N11c, N21c\cr
:\cr
O2,SPEC2a, N12a, N22a\cr
O3,SPEC2b, N12b, N22b\cr
O3,SPEC2c, N12c, N22c\cr
:}\cr
  where
\itemize{
\item Oi = code signifying which observer is primary for observation stop i
\item SPECix = 4-character code for species x for observation stop i
\item N1ix = Number of birds of species x observed by the primary observer at stop i
\item N2ix = Number of birds of species x observed by the secondary observer at stop i which were not observed by the primary observer
}

\strong{Data input (Independent observers)}

The input file for independent observers consists of counts, by species, of individuals observed by
 observer 1 which were not observed by observer 2, counts of individuals observed by observer 2,
 but not observed by observer 2, and counts of individuals observed by both observes. The input
 file should be a csv file in the following form:\cr

\code{O1,SPEC1a, N11a, N21a, N31a\cr
O1,SPEC1b, N11b, N21b, N31b\cr
:\cr
}\cr
 where
\itemize{
\item Oi = dummy code (ignored by program),
\item SPECix = 4-character code for species x for observation stop i,
\item N1ix = Number of birds of species x observed by observer 1, not observed by observer 2
\item N2ix = Number of birds of species x observed by observer 2, not observed by observer 1
\item N3ix = Number of birds of species x observed by both observers
}

\strong{Variance-Covariance Problems:}

When estimates of p1 or p2 for a species are zero or one, it is not possible to obtain a variance
for that species. This will be indicated by a message in the SURVIV output, and in the final output
list if the chosen model has this problem. If you are only interested in the estimates of p or N,
then this is not a problem. If you need the variances, you have two choices:

\itemize{
  \item Choose another model,
  \item Pool the offending species with another species or group of species.
}

\tabular{ll}{
Package: \tab dobserv\cr
Type: \tab Package\cr
Version: \tab 0.1.2\cr
Date: \tab 2017-01-30\cr
License: \tab GPL-2\cr
}
}
\examples{
\dontrun{
 #  dependent observers example... using BBS data...
fname=system.file('extdata','sample.csv',package='dobserv')
data=read.csv(fname,header=F,skip=1)            #  read data, skipping header line
pooled_data=pool_lt10(data)                     #  pool species with < 10 observations

#   pool CAGO,PUMA even though > 10 detections because all detections were by same obs.
i=which(pooled_data$sp=="CAGO" | pooled_data$sp=="PUMA"); pooled_data$sp[i]='grp1'

#  summarize by species...
sp=as.factor(pooled_data$sp); usp=levels(sp); nsp=length(usp); d2=matrix(0,2*nsp,2)
jj=(as.numeric(sp)-1)*2+(pooled_data[,1]==pooled_data[1,1])+1
for (i in 1:nrow(pooled_data)) d2[jj[i],]=d2[jj[i],]+as.numeric(pooled_data[i,3:4])
d2=data.frame(obs=rep(1:2,nsp),sp=rep(as.character(usp),each=2),X1=d2[,1],X01=d2[,2])

 #   run some models...
 #      ~obs means detection depends only on observer,
 #      ~sp means detection depends only on species,
 #      ~xxxx means detection depends on input covariate xxxx

z1=dobserv_dep_obs(d2,mods=c(~obs,~-1+sp,~sp+obs,~sp*obs))
#   z1 is a list with output from each of the models above (plus default model, ~1)

#  print results from model, p(sp*obs) (ie., detection different for each
#                                         species and observer)
print(round(z1$m$`p(sp * obs)`$z,4))

#-------------------------------
#  independent observer example with covariate...
fname=system.file('extdata','sample3.csv',package='dobserv')

 #  read sample data file which includes a covariate (temperature) in the last column
d3=read.csv(fname,header=F,skip=1); colnames(d3)=c('obs','sp','X10','X01','X11','temp')
 #  run a few models...
z2=dobserv_indep_obs(d3,mods=c(~obs,~temp*obs))
 #  print estimates for each species using model, p(temp*obs)...
print(z2$m$`p(temp * obs)`$z)
 #  print estimate of total number of individuals...
print(z2$m$`p(temp * obs)`$ntotal)

}
}
\author{
Jim Hines
}
